@@ -10,12 +10,11 @@ from typing import Literal, overload
 
 from parted import Device, Disk, DiskException, FileSystem, Geometry, IOException, Partition, PartitionException, freshDisk, getAllDevices, getDevice, newDisk
 
-from ..exceptions import DiskError, UnknownFilesystemFormat
-from ..general import SysCallError, SysCommand, SysCommandWorker
+from ..exceptions import DiskError, SysCallError, UnknownFilesystemFormat
+from ..general import SysCommand, SysCommandWorker
 from ..luks import Luks2
-from ..output import debug, error, info, log, warn
-from ..utils.util import is_subpath
-from .device_model import (
+from ..models.device import (
+	DEFAULT_ITER_TIME,
 	BDevice,
 	BtrfsMountOption,
 	DeviceModification,
@@ -39,10 +38,15 @@ from .device_model import (
 	_BtrfsSubvolumeInfo,
 	_DeviceInfo,
 	_PartitionInfo,
+)
+from ..models.users import Password
+from ..output import debug, error, info, log
+from ..utils.util import is_subpath
+from .utils import (
 	find_lsblk_info,
 	get_all_lsblk_info,
 	get_lsblk_info,
-	get_lsblk_output,
+	umount,
 )
 
 
@@ -51,12 +55,17 @@ class DeviceHandler:
 
 	def __init__(self) -> None:
 		self._devices: dict[Path, BDevice] = {}
+		self._partition_table = PartitionTable.default()
 		self.load_devices()
 
 	@property
 	def devices(self) -> list[BDevice]:
 		return list(self._devices.values())
 
+	@property
+	def partition_table(self) -> PartitionTable:
+		return self._partition_table
+
 	def load_devices(self) -> None:
 		block_devices = {}
 
@@ -85,7 +94,7 @@ class DeviceHandler:
 				if dev_lsblk_info.pttype:
 					disk = newDisk(device)
 				else:
-					disk = freshDisk(device, PartitionTable.GPT.value)
+					disk = freshDisk(device, self.partition_table.value)
 			except DiskException as err:
 				debug(f'Unable to get disk from {device.path}: {err}')
 				continue
@@ -111,8 +120,8 @@ class DeviceHandler:
 						partition,
 						lsblk_info,
 						fs_type,
-						subvol_infos
-					)
+						subvol_infos,
+					),
 				)
 
 			block_device = BDevice(disk, device_info, partition_infos)
@@ -147,7 +156,7 @@ class DeviceHandler:
 	def _determine_fs_type(
 		self,
 		partition: Partition,
-		lsblk_info: LsblkInfo | None = None
+		lsblk_info: LsblkInfo | None = None,
 	) -> FilesystemType | None:
 		try:
 			if partition.fileSystem:
@@ -186,10 +195,7 @@ class DeviceHandler:
 	def get_unique_path_for_device(self, dev_path: Path) -> Path | None:
 		paths = Path('/dev/disk/by-id').glob('*')
 		linked_targets = {p.resolve(): p for p in paths}
-		linked_wwn_targets = {
-			p: linked_targets[p] for p in linked_targets
-			if p.name.startswith('wwn-') or p.name.startswith('nvme-eui.')
-		}
+		linked_wwn_targets = {p: linked_targets[p] for p in linked_targets if p.name.startswith('wwn-') or p.name.startswith('nvme-eui.')}
 
 		if dev_path in linked_wwn_targets:
 			return linked_wwn_targets[dev_path]
@@ -206,7 +212,7 @@ class DeviceHandler:
 	def get_btrfs_info(
 		self,
 		dev_path: Path,
-		lsblk_info: LsblkInfo | None = None
+		lsblk_info: LsblkInfo | None = None,
 	) -> list[_BtrfsSubvolumeInfo]:
 		if not lsblk_info:
 			lsblk_info = get_lsblk_info(dev_path)
@@ -218,8 +224,8 @@ class DeviceHandler:
 			mountpoint = self._TMP_BTRFS_MOUNT
 		else:
 			# when multiple subvolumes are mounted then the lsblk output may look like
-			# "mountpoint": "/mnt/archinstall/.snapshots"
-			# "mountpoints": ["/mnt/archinstall/.snapshots", "/mnt/archinstall/home", ..]
+			# "mountpoint": "/mnt/archinstall/var/log"
+			# "mountpoints": ["/mnt/archinstall/var/log", "/mnt/archinstall/home", ..]
 			# so we'll determine the minimum common path and assume that's the root
 			try:
 				common_path = os.path.commonpath(lsblk_info.mountpoints)
@@ -250,7 +256,7 @@ class DeviceHandler:
 			subvol_infos.append(_BtrfsSubvolumeInfo(name, sub_vol_mountpoint))
 
 		if not lsblk_info.mountpoint:
-			self.umount(dev_path)
+			umount(dev_path)
 
 		return subvol_infos
 
@@ -258,7 +264,7 @@ class DeviceHandler:
 		self,
 		fs_type: FilesystemType,
 		path: Path,
-		additional_parted_options: list[str] = []
+		additional_parted_options: list[str] = [],
 	) -> None:
 		mkfs_type = fs_type.value
 		command = None
@@ -271,7 +277,7 @@ class DeviceHandler:
 			case FilesystemType.Ext2 | FilesystemType.Ext3 | FilesystemType.Ext4:
 				# Force create
 				options.append('-F')
-			case FilesystemType.Fat16 | FilesystemType.Fat32:
+			case FilesystemType.Fat12 | FilesystemType.Fat16 | FilesystemType.Fat32:
 				mkfs_type = 'fat'
 				# Set FAT size
 				options.extend(('-F', fs_type.value.removeprefix(mkfs_type)))
@@ -279,7 +285,7 @@ class DeviceHandler:
 				# Skip zeroing and bad sector check
 				options.append('--fast')
 			case FilesystemType.LinuxSwap:
-				command = "mkswap"
+				command = 'mkswap'
 			case _:
 				raise UnknownFilesystemFormat(f'Filetype "{fs_type.value}" is not supported')
 
@@ -301,16 +307,19 @@ class DeviceHandler:
 		self,
 		dev_path: Path,
 		mapper_name: str | None,
-		enc_password: str,
-		lock_after_create: bool = True
+		enc_password: Password | None,
+		lock_after_create: bool = True,
+		iter_time: int = DEFAULT_ITER_TIME,
 	) -> Luks2:
 		luks_handler = Luks2(
 			dev_path,
 			mapper_name=mapper_name,
-			password=enc_password
+			password=enc_password,
 		)
 
-		key_file = luks_handler.encrypt()
+		key_file = luks_handler.encrypt(iter_time=iter_time)
+
+		self.udev_sync()
 
 		luks_handler.unlock(key_file=key_file)
 
@@ -328,15 +337,20 @@ class DeviceHandler:
 		dev_path: Path,
 		mapper_name: str | None,
 		fs_type: FilesystemType,
-		enc_conf: DiskEncryption
+		enc_conf: DiskEncryption,
 	) -> None:
+		if not enc_conf.encryption_password:
+			raise ValueError('No encryption password provided')
+
 		luks_handler = Luks2(
 			dev_path,
 			mapper_name=mapper_name,
-			password=enc_conf.encryption_password
+			password=enc_conf.encryption_password,
 		)
 
-		key_file = luks_handler.encrypt()
+		key_file = luks_handler.encrypt(iter_time=enc_conf.iter_time)
+
+		self.udev_sync()
 
 		luks_handler.unlock(key_file=key_file)
 
@@ -352,7 +366,7 @@ class DeviceHandler:
 	def _lvm_info(
 		self,
 		cmd: str,
-		info_type: Literal['lv', 'vg', 'pvseg']
+		info_type: Literal['lv', 'vg', 'pvseg'],
 	) -> LvmVolumeInfo | LvmGroupInfo | LvmPVInfo | None:
 		raw_info = SysCommand(cmd).decode().split('\n')
 
@@ -381,32 +395,29 @@ class DeviceHandler:
 					return LvmVolumeInfo(
 						lv_name=entry['lv_name'],
 						vg_name=entry['vg_name'],
-						lv_size=Size(int(entry['lv_size'][:-1]), Unit.B, SectorSize.default())
+						lv_size=Size(int(entry['lv_size'][:-1]), Unit.B, SectorSize.default()),
 					)
 				case 'vg':
 					return LvmGroupInfo(
 						vg_uuid=entry['vg_uuid'],
-						vg_size=Size(int(entry['vg_size'][:-1]), Unit.B, SectorSize.default())
+						vg_size=Size(int(entry['vg_size'][:-1]), Unit.B, SectorSize.default()),
 					)
 
 		return None
 
 	@overload
-	def _lvm_info_with_retry(self, cmd: str, info_type: Literal['lv']) -> LvmVolumeInfo | None:
-		...
+	def _lvm_info_with_retry(self, cmd: str, info_type: Literal['lv']) -> LvmVolumeInfo | None: ...
 
 	@overload
-	def _lvm_info_with_retry(self, cmd: str, info_type: Literal['vg']) -> LvmGroupInfo | None:
-		...
+	def _lvm_info_with_retry(self, cmd: str, info_type: Literal['vg']) -> LvmGroupInfo | None: ...
 
 	@overload
-	def _lvm_info_with_retry(self, cmd: str, info_type: Literal['pvseg']) -> LvmPVInfo | None:
-		...
+	def _lvm_info_with_retry(self, cmd: str, info_type: Literal['pvseg']) -> LvmPVInfo | None: ...
 
 	def _lvm_info_with_retry(
 		self,
 		cmd: str,
-		info_type: Literal['lv', 'vg', 'pvseg']
+		info_type: Literal['lv', 'vg', 'pvseg'],
 	) -> LvmVolumeInfo | LvmGroupInfo | LvmPVInfo | None:
 		while True:
 			try:
@@ -415,31 +426,17 @@ class DeviceHandler:
 				time.sleep(3)
 
 	def lvm_vol_info(self, lv_name: str) -> LvmVolumeInfo | None:
-		cmd = (
-			'lvs --reportformat json '
-			'--unit B '
-			f'-S lv_name={lv_name}'
-		)
+		cmd = f'lvs --reportformat json --unit B -S lv_name={lv_name}'
 
 		return self._lvm_info_with_retry(cmd, 'lv')
 
 	def lvm_group_info(self, vg_name: str) -> LvmGroupInfo | None:
-		cmd = (
-			'vgs --reportformat json '
-			'--unit B '
-			'-o vg_name,vg_uuid,vg_size '
-			f'-S vg_name={vg_name}'
-		)
+		cmd = f'vgs --reportformat json --unit B -o vg_name,vg_uuid,vg_size -S vg_name={vg_name}'
 
 		return self._lvm_info_with_retry(cmd, 'vg')
 
 	def lvm_pvseg_info(self, vg_name: str, lv_name: str) -> LvmPVInfo | None:
-		cmd = (
-			'pvs '
-			'--segments -o+lv_name,vg_name '
-			f'-S vg_name={vg_name},lv_name={lv_name} '
-			'--reportformat json '
-		)
+		cmd = f'pvs --segments -o+lv_name,vg_name -S vg_name={vg_name},lv_name={lv_name} --reportformat json '
 
 		return self._lvm_info_with_retry(cmd, 'pvseg')
 
@@ -510,7 +507,7 @@ class DeviceHandler:
 		part_mod: PartitionModification,
 		block_device: BDevice,
 		disk: Disk,
-		requires_delete: bool
+		requires_delete: bool,
 	) -> None:
 		# when we require a delete and the partition to be (re)created
 		# already exists then we have to delete it first
@@ -528,18 +525,18 @@ class DeviceHandler:
 
 		start_sector = part_mod.start.convert(
 			Unit.sectors,
-			block_device.device_info.sector_size
+			block_device.device_info.sector_size,
 		)
 
 		length_sector = part_mod.length.convert(
 			Unit.sectors,
-			block_device.device_info.sector_size
+			block_device.device_info.sector_size,
 		)
 
 		geometry = Geometry(
 			device=block_device.disk.device,
 			start=start_sector.value,
-			length=length_sector.value
+			length=length_sector.value,
 		)
 
 		fs_value = part_mod.safe_fs_type.parted_value
@@ -549,7 +546,7 @@ class DeviceHandler:
 			disk=disk,
 			type=part_mod.type.get_partition_code(),
 			fs=filesystem,
-			geometry=geometry
+			geometry=geometry,
 		)
 
 		for flag in part_mod.flags:
@@ -588,7 +585,7 @@ class DeviceHandler:
 			debug(f'Unable to determine new uuid: {path}\n{lsblk_info}')
 			raise DiskError(f'Unable to determine new uuid: {path}')
 
-		debug(f'partition information found: {lsblk_info.json()}')
+		debug(f'partition information found: {lsblk_info.model_dump_json()}')
 
 		return lsblk_info
 
@@ -596,7 +593,7 @@ class DeviceHandler:
 		self,
 		path: Path,
 		btrfs_subvols: list[SubvolumeModification],
-		mount_options: list[str]
+		mount_options: list[str],
 	) -> None:
 		info(f'Creating subvolumes: {path}')
 
@@ -607,7 +604,7 @@ class DeviceHandler:
 
 			subvol_path = self._TMP_BTRFS_MOUNT / sub_vol.name
 
-			SysCommand(f"btrfs subvolume create -p {subvol_path}")
+			SysCommand(f'btrfs subvolume create -p {subvol_path}')
 
 			if BtrfsMountOption.nodatacow.value in mount_options:
 				try:
@@ -621,12 +618,12 @@ class DeviceHandler:
 				except SysCallError as err:
 					raise DiskError(f'Could not set compress attribute at {subvol_path}: {err}')
 
-		self.umount(path)
+		umount(path)
 
 	def create_btrfs_volumes(
 		self,
 		part_mod: PartitionModification,
-		enc_conf: 'DiskEncryption | None' = None
+		enc_conf: DiskEncryption | None = None,
 	) -> None:
 		info(f'Creating subvolumes: {part_mod.safe_dev_path}')
 
@@ -638,7 +635,7 @@ class DeviceHandler:
 			luks_handler = self.unlock_luks2_dev(
 				part_mod.safe_dev_path,
 				part_mod.mapper_name,
-				enc_conf.encryption_password
+				enc_conf.encryption_password,
 			)
 
 			if not luks_handler.mapper_dev:
@@ -653,7 +650,7 @@ class DeviceHandler:
 			dev_path,
 			self._TMP_BTRFS_MOUNT,
 			create_target_mountpoint=True,
-			options=part_mod.mount_options
+			options=part_mod.mount_options,
 		)
 
 		for sub_vol in sorted(part_mod.btrfs_subvols, key=lambda x: x.name):
@@ -661,22 +658,24 @@ class DeviceHandler:
 
 			subvol_path = self._TMP_BTRFS_MOUNT / sub_vol.name
 
-			SysCommand(f"btrfs subvolume create -p {subvol_path}")
+			SysCommand(f'btrfs subvolume create -p {subvol_path}')
 
-		self.umount(dev_path)
+		umount(dev_path)
 
 		if luks_handler is not None and luks_handler.mapper_dev is not None:
 			luks_handler.lock()
 
-	def unlock_luks2_dev(self, dev_path: Path, mapper_name: str, enc_password: str) -> Luks2:
+	def unlock_luks2_dev(
+		self,
+		dev_path: Path,
+		mapper_name: str,
+		enc_password: Password | None,
+	) -> Luks2:
 		luks_handler = Luks2(dev_path, mapper_name=mapper_name, password=enc_password)
 
 		if not luks_handler.is_unlocked():
 			luks_handler.unlock()
 
-		if not luks_handler.is_unlocked():
-			raise DiskError(f'Failed to unlock luks2 device: {dev_path}')
-
 		return luks_handler
 
 	def umount_all_existing(self, device_path: Path) -> None:
@@ -691,28 +690,25 @@ class DeviceHandler:
 			if partition.fs_type == FilesystemType.Crypto_luks:
 				Luks2(partition.path).lock()
 			else:
-				self.umount(partition.path, recursive=True)
+				umount(partition.path, recursive=True)
 
 	def partition(
 		self,
 		modification: DeviceModification,
-		partition_table: PartitionTable | None = None
+		partition_table: PartitionTable | None = None,
 	) -> None:
 		"""
 		Create a partition table on the block device and create all partitions.
 		"""
-		if modification.wipe:
-			if partition_table is None:
-				raise ValueError('Modification is marked as wipe but no partitioning table was provided')
-
-			if partition_table.MBR and len(modification.partitions) > 3:
-				raise DiskError('Too many partitions on disk, MBR disks can only have 3 primary partitions')
+		partition_table = partition_table or self.partition_table
 
 		# WARNING: the entire device will be wiped and all data lost
 		if modification.wipe:
+			if partition_table.is_mbr() and len(modification.partitions) > 3:
+				raise DiskError('Too many partitions on disk, MBR disks can only have 3 primary partitions')
+
 			self.wipe_dev(modification.device)
-			part_table = partition_table.value if partition_table else None
-			disk = freshDisk(modification.device.disk.device, part_table)
+			disk = freshDisk(modification.device.disk.device, partition_table.value)
 		else:
 			info(f'Use existing device: {modification.device_path}')
 			disk = modification.device.disk
@@ -743,7 +739,7 @@ class DeviceHandler:
 		target_mountpoint: Path,
 		mount_fs: str | None = None,
 		create_target_mountpoint: bool = True,
-		options: list[str] = []
+		options: list[str] = [],
 	) -> None:
 		if create_target_mountpoint and not target_mountpoint.exists():
 			target_mountpoint.mkdir(parents=True, exist_ok=True)
@@ -774,23 +770,6 @@ class DeviceHandler:
 		except SysCallError as err:
 			raise DiskError(f'Could not mount {dev_path}: {command}\n{err.message}')
 
-	def umount(self, mountpoint: Path, recursive: bool = False) -> None:
-		lsblk_info = get_lsblk_info(mountpoint)
-
-		if not lsblk_info.mountpoints:
-			return
-
-		debug(f'Partition {mountpoint} is currently mounted at: {[str(m) for m in lsblk_info.mountpoints]}')
-
-		cmd = ['umount']
-
-		if recursive:
-			cmd.append('-R')
-
-		for path in lsblk_info.mountpoints:
-			debug(f'Unmounting mountpoint: {path}')
-			SysCommand(cmd + [str(path)])
-
 	def detect_pre_mounted_mods(self, base_mountpoint: Path) -> list[DeviceModification]:
 		part_mods: dict[Path, list[PartitionModification]] = {}
 
@@ -828,7 +807,7 @@ class DeviceHandler:
 			SysCommand(command)
 		except SysCallError as err:
 			if 'have been written, but we have been unable to inform the kernel of the change' in str(err):
-				log(f"Partprobe was not able to inform the kernel of the new disk state (ignoring error): {err}", fg="gray", level=logging.INFO)
+				log(f'Partprobe was not able to inform the kernel of the new disk state (ignoring error): {err}', fg='gray', level=logging.INFO)
 			else:
 				error(f'"{command}" failed to run (continuing anyway): {err}')
 
@@ -848,6 +827,7 @@ class DeviceHandler:
 		auto-discovery tools don't recognize anything here.
 		"""
 		info(f'Wiping partitions and metadata: {block_device.device_info.path}')
+
 		for partition in block_device.partition_infos:
 			luks = Luks2(partition.path)
 			if luks.isLuks():
@@ -866,13 +846,3 @@ class DeviceHandler:
 
 
 device_handler = DeviceHandler()
-
-
-def disk_layouts() -> str:
-	try:
-		lsblk_output = get_lsblk_output()
-	except SysCallError as err:
-		warn(f"Could not return disk layouts: {err}")
-		return ''
-
-	return lsblk_output.model_dump_json(indent=4)
