@@ -1,8 +1,9 @@
 import time
 import urllib.parse
 from pathlib import Path
-from typing import TYPE_CHECKING, override
+from typing import override
 
+from archinstall.lib.translationhandler import tr
 from archinstall.tui.curses_menu import EditMenu, SelectMenu, Tui
 from archinstall.tui.menu_item import MenuItem, MenuItemGroup
 from archinstall.tui.result import ResultType
@@ -24,27 +25,20 @@ from .models.packages import Repository
 from .networking import fetch_data_from_url
 from .output import FormattedOutput, debug
 
-if TYPE_CHECKING:
-	from collections.abc import Callable
-
-	from archinstall.lib.translationhandler import DeferredTranslation
-
-	_: Callable[[str], DeferredTranslation]
-
 
 class CustomMirrorRepositoriesList(ListManager[CustomRepository]):
 	def __init__(self, custom_repositories: list[CustomRepository]):
 		self._actions = [
-			str(_('Add a custom repository')),
-			str(_('Change custom repository')),
-			str(_('Delete custom repository'))
+			tr('Add a custom repository'),
+			tr('Change custom repository'),
+			tr('Delete custom repository'),
 		]
 
 		super().__init__(
 			custom_repositories,
 			[self._actions[0]],
 			self._actions[1:],
-			''
+			'',
 		)
 
 	@override
@@ -56,7 +50,7 @@ class CustomMirrorRepositoriesList(ListManager[CustomRepository]):
 		self,
 		action: str,
 		entry: CustomRepository | None,
-		data: list[CustomRepository]
+		data: list[CustomRepository],
 	) -> list[CustomRepository]:
 		if action == self._actions[0]:  # add
 			new_repo = self._add_custom_repository()
@@ -75,10 +69,10 @@ class CustomMirrorRepositoriesList(ListManager[CustomRepository]):
 
 	def _add_custom_repository(self, preset: CustomRepository | None = None) -> CustomRepository | None:
 		edit_result = EditMenu(
-			str(_('Repository name')),
+			tr('Repository name'),
 			alignment=Alignment.CENTER,
 			allow_skip=True,
-			default_text=preset.name if preset else None
+			default_text=preset.name if preset else None,
 		).input()
 
 		match edit_result.type_:
@@ -89,14 +83,14 @@ class CustomMirrorRepositoriesList(ListManager[CustomRepository]):
 			case _:
 				raise ValueError('Unhandled return type')
 
-		header = f'{_("Name")}: {name}'
+		header = f'{tr("Name")}: {name}'
 
 		edit_result = EditMenu(
-			str(_('Url')),
+			tr('Url'),
 			header=header,
 			alignment=Alignment.CENTER,
 			allow_skip=True,
-			default_text=preset.url if preset else None
+			default_text=preset.url if preset else None,
 		).input()
 
 		match edit_result.type_:
@@ -107,8 +101,8 @@ class CustomMirrorRepositoriesList(ListManager[CustomRepository]):
 			case _:
 				raise ValueError('Unhandled return type')
 
-		header += f'\n{_("Url")}: {url}\n'
-		prompt = f'{header}\n' + str(_('Select signature check'))
+		header += f'\n{tr("Url")}: {url}\n'
+		prompt = f'{header}\n' + tr('Select signature check')
 
 		sign_chk_items = [MenuItem(s.value, value=s.value) for s in SignCheck]
 		group = MenuItemGroup(sign_chk_items, sort_items=False)
@@ -120,7 +114,7 @@ class CustomMirrorRepositoriesList(ListManager[CustomRepository]):
 			group,
 			header=prompt,
 			alignment=Alignment.CENTER,
-			allow_skip=False
+			allow_skip=False,
 		).run()
 
 		match result.type_:
@@ -129,7 +123,7 @@ class CustomMirrorRepositoriesList(ListManager[CustomRepository]):
 			case _:
 				raise ValueError('Unhandled return type')
 
-		header += f'{_("Signature check")}: {sign_check.value}\n'
+		header += f'{tr("Signature check")}: {sign_check.value}\n'
 		prompt = f'{header}\n' + 'Select signature option'
 
 		sign_opt_items = [MenuItem(s.value, value=s.value) for s in SignOption]
@@ -142,7 +136,7 @@ class CustomMirrorRepositoriesList(ListManager[CustomRepository]):
 			group,
 			header=prompt,
 			alignment=Alignment.CENTER,
-			allow_skip=False
+			allow_skip=False,
 		).run()
 
 		match result.type_:
@@ -157,16 +151,16 @@ class CustomMirrorRepositoriesList(ListManager[CustomRepository]):
 class CustomMirrorServersList(ListManager[CustomServer]):
 	def __init__(self, custom_servers: list[CustomServer]):
 		self._actions = [
-			str(_('Add a custom server')),
-			str(_('Change custom server')),
-			str(_('Delete custom server'))
+			tr('Add a custom server'),
+			tr('Change custom server'),
+			tr('Delete custom server'),
 		]
 
 		super().__init__(
 			custom_servers,
 			[self._actions[0]],
 			self._actions[1:],
-			''
+			'',
 		)
 
 	@override
@@ -178,7 +172,7 @@ class CustomMirrorServersList(ListManager[CustomServer]):
 		self,
 		action: str,
 		entry: CustomServer | None,
-		data: list[CustomServer]
+		data: list[CustomServer],
 	) -> list[CustomServer]:
 		if action == self._actions[0]:  # add
 			new_server = self._add_custom_server()
@@ -197,10 +191,10 @@ class CustomMirrorServersList(ListManager[CustomServer]):
 
 	def _add_custom_server(self, preset: CustomServer | None = None) -> CustomServer | None:
 		edit_result = EditMenu(
-			str(_('Server url')),
+			tr('Server url'),
 			alignment=Alignment.CENTER,
 			allow_skip=True,
-			default_text=preset.url if preset else None
+			default_text=preset.url if preset else None,
 		).input()
 
 		match edit_result.type_:
@@ -216,7 +210,7 @@ class CustomMirrorServersList(ListManager[CustomServer]):
 class MirrorMenu(AbstractSubMenu[MirrorConfiguration]):
 	def __init__(
 		self,
-		preset: MirrorConfiguration | None = None
+		preset: MirrorConfiguration | None = None,
 	):
 		if preset:
 			self._mirror_config = preset
@@ -229,42 +223,42 @@ class MirrorMenu(AbstractSubMenu[MirrorConfiguration]):
 		super().__init__(
 			self._item_group,
 			config=self._mirror_config,
-			allow_reset=True
+			allow_reset=True,
 		)
 
 	def _define_menu_options(self) -> list[MenuItem]:
 		return [
 			MenuItem(
-				text=str(_('Select regions')),
+				text=tr('Select regions'),
 				action=select_mirror_regions,
 				value=self._mirror_config.mirror_regions,
 				preview_action=self._prev_regions,
-				key='mirror_regions'
+				key='mirror_regions',
 			),
 			MenuItem(
-				text=str(_('Add custom servers')),
+				text=tr('Add custom servers'),
 				action=add_custom_mirror_servers,
 				value=self._mirror_config.custom_servers,
 				preview_action=self._prev_custom_servers,
-				key='custom_servers'
+				key='custom_servers',
 			),
 			MenuItem(
-				text=str(_('Optional repositories')),
+				text=tr('Optional repositories'),
 				action=select_optional_repositories,
 				value=[],
 				preview_action=self._prev_additional_repos,
-				key='optional_repositories'
+				key='optional_repositories',
 			),
 			MenuItem(
-				text=str(_('Add custom repository')),
+				text=tr('Add custom repository'),
 				action=select_custom_mirror,
 				value=self._mirror_config.custom_repositories,
 				preview_action=self._prev_custom_mirror,
-				key='custom_repositories'
-			)
+				key='custom_repositories',
+			),
 		]
 
-	def _prev_regions(self, item: MenuItem) -> str | None:
+	def _prev_regions(self, item: MenuItem) -> str:
 		regions = item.get_value()
 
 		output = ''
@@ -282,7 +276,7 @@ class MirrorMenu(AbstractSubMenu[MirrorConfiguration]):
 		if item.value:
 			repositories: list[Repository] = item.value
 			repos = ', '.join([repo.value for repo in repositories])
-			return f'{_("Additional repositories")}: {repos}'
+			return f'{tr("Additional repositories")}: {repos}'
 		return None
 
 	def _prev_custom_mirror(self, item: MenuItem) -> str | None:
@@ -302,13 +296,13 @@ class MirrorMenu(AbstractSubMenu[MirrorConfiguration]):
 		return output.strip()
 
 	@override
-	def run(self) -> MirrorConfiguration:
-		super().run()
+	def run(self, additional_title: str | None = None) -> MirrorConfiguration:
+		super().run(additional_title=additional_title)
 		return self._mirror_config
 
 
 def select_mirror_regions(preset: list[MirrorRegion]) -> list[MirrorRegion]:
-	Tui.print(str(_('Loading mirror regions...')), clear_screen=True)
+	Tui.print(tr('Loading mirror regions...'), clear_screen=True)
 
 	mirror_list_handler.load_mirrors()
 	available_regions = mirror_list_handler.get_mirror_regions()
@@ -326,7 +320,7 @@ def select_mirror_regions(preset: list[MirrorRegion]) -> list[MirrorRegion]:
 	result = SelectMenu[MirrorRegion](
 		group,
 		alignment=Alignment.CENTER,
-		frame=FrameProperties.min(str(_('Mirror regions'))),
+		frame=FrameProperties.min(tr('Mirror regions')),
 		allow_reset=True,
 		allow_skip=True,
 		multi=True,
@@ -342,12 +336,12 @@ def select_mirror_regions(preset: list[MirrorRegion]) -> list[MirrorRegion]:
 			return selected_mirrors
 
 
-def add_custom_mirror_servers(preset: list[CustomServer] = []):
+def add_custom_mirror_servers(preset: list[CustomServer] = []) -> list[CustomServer]:
 	custom_mirrors = CustomMirrorServersList(preset).run()
 	return custom_mirrors
 
 
-def select_custom_mirror(preset: list[CustomRepository] = []):
+def select_custom_mirror(preset: list[CustomRepository] = []) -> list[CustomRepository]:
 	custom_mirrors = CustomMirrorRepositoriesList(preset).run()
 	return custom_mirrors
 
@@ -371,7 +365,7 @@ def select_optional_repositories(preset: list[Repository]) -> list[Repository]:
 		frame=FrameProperties.min('Additional repositories'),
 		allow_reset=True,
 		allow_skip=True,
-		multi=True
+		multi=True,
 	).run()
 
 	match result.type_:
@@ -411,6 +405,7 @@ class MirrorListHandler:
 
 	def load_mirrors(self) -> None:
 		from .args import arch_config_handler
+
 		if arch_config_handler.args.offline:
 			self.load_local_mirrors()
 		else:
@@ -418,7 +413,7 @@ class MirrorListHandler:
 				self.load_local_mirrors()
 
 	def load_remote_mirrors(self) -> bool:
-		url = "https://archlinux.org/mirrors/status/json/"
+		url = 'https://archlinux.org/mirrors/status/json/'
 		attempts = 3
 
 		for attempt_nr in range(attempts):
@@ -443,40 +438,41 @@ class MirrorListHandler:
 		region_list = mappings[region]
 		return sorted(region_list, key=lambda mirror: (mirror.score, mirror.speed))
 
-	def _parse_remote_mirror_list(self, mirrorlist: str) -> dict[str, list[MirrorStatusEntryV3]] | None:
+	def _parse_remote_mirror_list(self, mirrorlist: str) -> dict[str, list[MirrorStatusEntryV3]]:
 		mirror_status = MirrorStatusListV3.model_validate_json(mirrorlist)
 
 		sorting_placeholder: dict[str, list[MirrorStatusEntryV3]] = {}
 
 		for mirror in mirror_status.urls:
 			# We filter out mirrors that have bad criteria values
-			if any([
-				mirror.active is False,  # Disabled by mirror-list admins
-				mirror.last_sync is None,  # Has not synced recently
-				# mirror.score (error rate) over time reported from backend:
-				# https://github.com/archlinux/archweb/blob/31333d3516c91db9a2f2d12260bd61656c011fd1/mirrors/utils.py#L111C22-L111C66
-				(mirror.score is None or mirror.score >= 100),
-			]):
+			if any(
+				[
+					mirror.active is False,  # Disabled by mirror-list admins
+					mirror.last_sync is None,  # Has not synced recently
+					# mirror.score (error rate) over time reported from backend:
+					# https://github.com/archlinux/archweb/blob/31333d3516c91db9a2f2d12260bd61656c011fd1/mirrors/utils.py#L111C22-L111C66
+					(mirror.score is None or mirror.score >= 100),
+				]
+			):
 				continue
 
-			if mirror.country == "":
+			if mirror.country == '':
 				# TODO: This should be removed once RFC!29 is merged and completed
 				# Until then, there are mirrors which lacks data in the backend
 				# and there is no way of knowing where they're located.
 				# So we have to assume world-wide
-				mirror.country = "Worldwide"
+				mirror.country = 'Worldwide'
 
 			if mirror.url.startswith('http'):
 				sorting_placeholder.setdefault(mirror.country, []).append(mirror)
 
-		sorted_by_regions: dict[str, list[MirrorStatusEntryV3]] = dict({
-			region: unsorted_mirrors
-			for region, unsorted_mirrors in sorted(sorting_placeholder.items(), key=lambda item: item[0])
-		})
+		sorted_by_regions: dict[str, list[MirrorStatusEntryV3]] = dict(
+			{region: unsorted_mirrors for region, unsorted_mirrors in sorted(sorting_placeholder.items(), key=lambda item: item[0])}
+		)
 
 		return sorted_by_regions
 
-	def _parse_locale_mirrors(self, mirrorlist: str) -> dict[str, list[MirrorStatusEntryV3]] | None:
+	def _parse_locale_mirrors(self, mirrorlist: str) -> dict[str, list[MirrorStatusEntryV3]]:
 		lines = mirrorlist.splitlines()
 
 		# remove empty lines
